import cv2  # 导入OpenCV库
import time  # 导入time库用于处理时间相关功能
import numpy as np  # 导入numpy库，用于处理数值计算
from pyzbar.pyzbar import decode  # 从pyzbar库导入decode函数，用于解码QR码

def main():  # 定义主函数
    cap = cv2.VideoCapture(0)  # 开启摄像头，'0'代表默认的摄像头
    while not cap.isOpened():  # 如果摄像头没有成功开启，则循环等待
        continue
    cap_w, cap_h = 240, 320  # 设置摄像头捕获的图像宽度和高度
    cv2.namedWindow("qrwindows", cv2.WND_PROP_FULLSCREEN)  # 创建一个名为'qrwindows'的窗口
    cv2.setWindowProperty("qrwindows", cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)  # 设置窗口'qrwindows'为全屏显示

    while True:  # 无限循环，直到程序被手动停止
        qrimg_success, qrimg_src = cap.read()  # 从摄像头读取一帧图像
        xmin, ymin, w, h = 250, 100, cap_w, cap_h  # 设置要处理的图像区域的位置和大小
        qrimg_src = qrimg_src[ymin:ymin+h, xmin:xmin+w]  # 从摄像头捕获的图像中裁剪出感兴趣的区域
        qrcode_val = decode(qrimg_src)  # 使用pyzbar的decode函数解码裁剪出的图像中的QR码

        if len(qrcode_val) > 0:  # 如果解码出至少一个QR码
            for i in range(len(qrcode_val)):  # 遍历所有解码出的QR码
                qrcode_str = qrcode_val[i][0].decode()  # 获取QR码的文本内容
                print(qrcode_str)  # 在控制台打印QR码的文本内容
                # 下面的代码获取QR码定位点的坐标，并绘制识别框和定位点
                point_x = qrcode_val[i][2][0]
                point_y = qrcode_val[i][2][1]
                point_w = qrcode_val[i][2][2]
                point_h = qrcode_val[i][2][3]
                point_1_x = qrcode_val[i][3][0][0]
                point_1_y = qrcode_val[i][3][0][1]
                point_2_x = qrcode_val[i][3][1][0]
                point_2_y = qrcode_val[i][3][1][1]
                point_3_x = qrcode_val[i][3][2][0]
                point_3_y = qrcode_val[i][3][2][1]
                point_4_x = qrcode_val[i][3][3][0]
                point_4_y = qrcode_val[i][3][3][1]
                # 绘制识别框
                cv2.rectangle(qrimg_src, (point_x, point_y), (point_x+point_w, point_y+point_h), (255, 0, 255), 2)
                # 绘制定位点之间的连线
                cv2.line(qrimg_src, (point_1_x, point_1_y), (point_2_x, point_2_y), (255, 0, 0), 2, cv2.FILLED)
                cv2.line(qrimg_src, (point_2_x, point_2_y), (point_3_x, point_3_y), (255, 0, 0), 2, cv2.FILLED)
                cv2.line(qrimg_src, (point_3_x, point_3_y), (point_4_x, point_4_y), (255, 0, 0), 2, cv2.FILLED)
                cv2.line(qrimg_src, (point_4_x, point_4_y), (point_1_x, point_1_y), (255, 0, 0), 2, cv2.FILLED)
                # 绘制定位点
                cv2.circle(qrimg_src, (point_1_x, point_1_y), 5, (255, 0, 0), 2)
                cv2.circle(qrimg_src, (point_2_x, point_2_y), 5, (0, 255, 0), 2)
                cv2.circle(qrimg_src, (point_3_x, point_3_y), 5, (0, 0, 255), 2)
                cv2.circle(qrimg_src, (point_4_x, point_4_y), 5, (255, 255, 0), 2)
                # 在识别框上方显示QR码的文本内容
                cv2.putText(qrimg_src, qrcode_str, (point_x, point_y-15), cv2.FONT_HERSHEY_COMPLEX, 0.4, (0, 0, 255), 1)
            cv2.imshow('qrwindows', qrimg_src)  # 在窗口'qrwindows'中显示带有识别框和文本的图像
            cv2.waitKey(10)  # 等待10毫秒，如果有按键事件则立即返回按键ASCII码，否则返回-1
            time.sleep(1)  # 等待1秒
        else:
            cv2.imshow('qrwindows', qrimg_src)  # 如果没有检测到QR码，则在窗口'qrwindows'中显示原始图像
            cv2.waitKey(10)  # 等待10毫秒，如果有按键事件则立即返回按键ASCII码，否则返回-1

if __name__ == '__main__':  # 如果此文件作为主程序运行
    main()  # 调用主函数
